@@ -1,6 +1,6 @@
 /*
  * C++ BMP library.
- * Copyright (C) 2006 - 2012 René Rebe, ExactCODE GmbH Germany
+ * Copyright (C) 2006 - 2013 René Rebe, ExactCODE GmbH Germany
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -227,7 +227,8 @@ struct BMPColorEntry
  */
 static void rearrangePixels(uint8_t* buf, uint32_t width, uint32_t bit_count)
 {
-  char tmp;
+  uint16_t tmp;
+  uint16_t* buf16 = (uint16_t*)buf;
   
   switch (bit_count) {
     
@@ -241,7 +242,15 @@ static void rearrangePixels(uint8_t* buf, uint32_t width, uint32_t bit_count)
       *(buf + 2) = tmp;
     }
     break;
-  
+
+  case 48:
+    for (uint32_t i = 0; i < width; i++, buf16 += 3) {
+      tmp = *buf16;
+      *buf16 = *(buf16 + 2);
+      *(buf16 + 2) = tmp;
+    }
+    break;
+
   case 32:
     {
       uint8_t* buf1 = buf;
@@ -339,8 +348,8 @@ int BMPCodec::readImageWithoutFileHeader (std::istream* stream, Image& image, co
     stream->read((char*)&info_hdr.iAlphaMask, 4);
     
     n_clr_elems = 4;
-    image.setResolution((2.54 * info_hdr.iXPelsPerMeter + .05) / 100,
-                        (2.54 * info_hdr.iYPelsPerMeter + .05) / 100);
+    image.setResolution((2.54 * info_hdr.iXPelsPerMeter) / 100 + .5,
+                        (2.54 * info_hdr.iYPelsPerMeter) / 100 + .5);
   }
   
   if (bmp_type == BMPT_OS22) {
@@ -374,6 +383,7 @@ int BMPCodec::readImageWithoutFileHeader (std::istream* stream, Image& image, co
   case 16:
   case 24:
   case 32:
+  case 48:
     break;
   default:
     std::cerr << "BMPCodec:: Cannot read " << info_hdr.iBitCount
@@ -435,7 +445,12 @@ int BMPCodec::readImageWithoutFileHeader (std::istream* stream, Image& image, co
       image.spp = 3;
       image.bps = 8;
       break;
-    
+
+    case 48:
+      image.spp = 3;
+      image.bps = 16;
+      break;
+
     default:
       break;
     }
@@ -643,7 +658,7 @@ int BMPCodec::readImageWithoutFileHeader (std::istream* stream, Image& image, co
     break;
   } /* switch */
   
-  // convert to RGB color-space - we do not handle palete images internally
+  // convert to RGB color-space - we do not handle palette images internally
   
   // no color table anyway or RGB* ?
   if (clr_tbl && image.spp < 3)
@@ -682,7 +697,7 @@ int BMPCodec::readImageWithoutFileHeader (std::istream* stream, Image& image, co
 bool BMPCodec::writeImage (std::ostream* stream, Image& image, int quality,
 			   const std::string& compress)
 {
-  if (image.bps > 8 || image.bps == 2 || image.spp > 3) {
+  if (image.bps > 16 || image.bps == 2 || image.spp > 3) {
     std::cerr << "BMPCodec: " << image.bps << " bits and "
 	      << image.spp << " samples not supported." << std::endl;
     return false;
@@ -713,8 +728,8 @@ bool BMPCodec::writeImage (std::ostream* stream, Image& image, int quality,
   info_hdr.iBitCount = image.spp * image.bps;
   info_hdr.iCompression = BMPC_RGB;
   info_hdr.iSizeImage  = image.stride()*image.h; // TODO: compressed size
-  info_hdr.iXPelsPerMeter = (int32_t) (image.resolutionX() * 100 / 2.54);
-  info_hdr.iYPelsPerMeter = (int32_t) (image.resolutionY() * 100 / 2.54);
+  info_hdr.iXPelsPerMeter = (int32_t) (100. * image.resolutionX() / 2.54 + .5);
+  info_hdr.iYPelsPerMeter = (int32_t) (100. * image.resolutionY() / 2.54 + .5);
   info_hdr.iClrUsed = image.spp == 1 ? 1 << image.bps : 0;
   info_hdr.iClrImportant = 0;
   info_hdr.iRedMask = 0;
