@@ -29,6 +29,46 @@
 
 #include "Endianess.hh"
 
+template <typename T>
+struct histogram_template
+{
+  std::vector<std::vector<unsigned int> > operator() (Image& image, int bins = 256)
+  {
+    std::vector<std::vector<unsigned int> > hist;
+    hist.resize(image.spp);
+
+    typename T::accu a;
+    
+      for (int i = 0; i < image.spp; ++i)
+	  hist[i].resize(bins, 0);
+      
+      typename T::accu one = T::accu::one();
+
+      T it (image);
+      for (int y = 0; y < image.h; ++y) {
+	for (int x = 0; x < image.w; ++x, ++it)
+	  {
+	    a = *it;
+            for (int i = 0; i < image.spp; ++i) {
+		typename T::accu::vtype v = a.v[i];
+    		int j = (int)(v * (bins-1) / one.v[i]);
+		if (j < 0) j = 0;
+		else if (j >= bins) j = bins - 1;
+		hist[i][j]++;
+	    }
+	  }
+      }
+
+      return hist;
+    }
+};
+
+std::vector<std::vector<unsigned int> > histogram(Image& image, int bins)
+{
+    return codegen_return<std::vector<std::vector<unsigned int> >,
+	histogram_template> (image, bins);
+}
+
 template <typename T>
 struct normalize_template
 {
@@ -142,16 +182,17 @@ void colorspace_argb8_to_rgb8 (Image& image)
   image.resize(image.w, image.h); // realloc
 }
 
-void colorspace_rgb8_to_gray8 (Image& image, const int bytes)
+void colorspace_rgb8_to_gray8 (Image& image, const int bytes, const int wR, const int wG, const int wB)
 {
+  const int sum = wR + wG + wB;
   uint8_t* output = image.getRawData();
   for (uint8_t* it = image.getRawData(); it < image.getRawData() + image.stride() * image.h; it += bytes)
     {
       // R G B order and associated weighting
-      int c = (int)it[0] * 28;
-      c += (int)it[1] * 59;
-      c += (int)it[2] * 11;
-      *output++ = (uint8_t)(c / 100);
+      int c  = (int)it[0] * wR;
+          c += (int)it[1] * wG;
+          c += (int)it[2] * wB;
+      *output++ = (uint8_t)(c / sum);
     }
   image.spp = 1; // converted data right now
   image.resize(image.w, image.h); // realloc
